/***************************************************************************
                          dictactions.cpp  -  description
                             -------------------
    begin                : . .. 22 20:37:33 ICT 2002
    copyright            : (C) 2002 by Thanomsub Noppaburana
    email                : donga_n@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlabel.h>
#include <qpushbutton.h>

#include <kcombobox.h>
#include <ktoolbar.h>

#include "dictactions.h"


// ***************************************************************
//                 Class CActLabel Implementation
// ***************************************************************

CActLabel::CActLabel(const QString &text, QObject *parent, const char *name)
  : KAction(text, 0, parent, name)
{
}


CActLabel::~CActLabel()
{
}


int CActLabel::plug(QWidget *widget, int index)
{
  if (widget->inherits("KToolBar")) {
    KToolBar *toolBar = (KToolBar *) widget;
    int btnId = KAction::getToolButtonID();

    QLabel *label = new QLabel(text(), widget);
    label->setMinimumWidth(label->sizeHint().width());
    label->setAlignment(AlignCenter | AlignVCenter);
    label->adjustSize();

    toolBar->insertWidget(btnId, label->width(), label, index);
    addContainer(toolBar, btnId);
    connect(toolBar, SIGNAL(destroyed()), this, SLOT(slotDestroyed()));
    labelPtr = label;

    return containerCount() - 1;
  }
  return -1;
}


void CActLabel::unplug(QWidget *widget)
{
  if (widget->inherits("KToolBar")) {
    KToolBar *toolBar = (KToolBar *) widget;
    int btnId = findContainer(toolBar);

    if (btnId != -1) {
      toolBar->removeItem(itemId(btnId));
      removeContainer(btnId);
    }
    return;
  }
}


void CActLabel::setBuddy(QWidget *buddy)
{
  if (labelPtr && buddy) labelPtr->setBuddy(buddy);
}



// ***************************************************************
//              Class CActComboBox Implementation
// ***************************************************************

CActComboBox::CActComboBox(const QString &text, QObject *parent,
                           const char *name, bool editable, bool autoSized)
  : KAction(text, 0, parent, name), flagEditable(editable), flagAutoSized(autoSized)
{
}


CActComboBox::~CActComboBox()
{
}


int CActComboBox::plug(QWidget *widget, int index)
{
  if (widget->inherits("KToolBar")) {
    KToolBar* toolBar = static_cast<KToolBar*>(widget);
    int btnId = KAction::getToolButtonID();
    comboPtr = new KComboBox(flagEditable, toolBar);

    toolBar->insertWidget(btnId, comboPtr->sizeHint().width(), comboPtr, index);
    toolBar->setItemAutoSized(btnId, flagAutoSized);

    if (comboPtr) {
      connect(toolBar->getCombo(btnId), SIGNAL(activated(const QString&)), SLOT(slotComboActivated(const QString&)));
      connect(toolBar->getCombo(btnId), SIGNAL(activated(int)), SLOT(slotComboActivated(int)));
      if (flagEditable) comboPtr->setInsertionPolicy(QComboBox::NoInsertion);
    }

    addContainer(toolBar, btnId);
    connect(toolBar, SIGNAL(destroyed()), this, SLOT(slotDestroyed()));
    return containerCount() - 1;
  }
  return -1;
}


void CActComboBox::unplug(QWidget *widget)
{
  if (widget->inherits("KToolBar")) {
    KToolBar *toolBar = (KToolBar *)widget;
    int btnId = findContainer(toolBar);

    if (btnId != -1) {
      toolBar->removeItem(itemId(btnId));
      removeContainer(btnId);
    }
    return;
  }
}


QWidget* CActComboBox::widget()
{
  return comboPtr;
}


void CActComboBox::setFocus()
{
  if (comboPtr) comboPtr->setFocus();
}


QString CActComboBox::currentText()
{
  if (comboPtr)
    return comboPtr->currentText();
  else
    return QString::null;
}


void CActComboBox::setEditText(const QString &s)
{
  if (comboPtr && flagEditable) comboPtr->setEditText(s);
}


void CActComboBox::setCurrentItem(int index)
{
  if (comboPtr) comboPtr->setCurrentItem(index);
}


void CActComboBox::clearEdit()
{
  if (comboPtr && flagEditable) comboPtr->clearEdit();
}


void CActComboBox::clear()
{
  if (comboPtr) {
    comboPtr->clear();
    if (flagEditable && comboPtr->completionObject()) comboPtr->completionObject()->clear();
  }
}


void CActComboBox::setList(QStringList items)
{
  if (comboPtr) {
    comboPtr->clear();
    comboPtr->insertStringList(items);
    if (flagEditable && comboPtr->completionObject()) comboPtr->completionObject()->setItems(items);
    if (!flagAutoSized) comboPtr->setFixedWidth(comboPtr->sizeHint().width());
  }
}

// ********* CActComboBox slot implement *********

void CActComboBox::slotComboActivated(int idx)
{
  emit(activated(idx));
}


void CActComboBox::slotComboActivated(const QString &str)
{
  emit(activated(str));
}


// ***************************************************************
//                   Class CActButton Implementation
// ***************************************************************

CActButton::CActButton(const QString& text, QObject* receiver,
                       const char* slot, QObject* parent, const char* name)
  : KAction(text, 0, receiver, slot, parent, name)
{
}


CActButton::~CActButton()
{
}


int CActButton::plug(QWidget *widget, int index)
{
  if (widget->inherits("KToolBar")) {
    KToolBar *toolBar = (KToolBar *) widget;
    int btnId = KAction::getToolButtonID();

    QPushButton *button = new QPushButton(text(), widget);
    button->adjustSize();
    connect(button, SIGNAL(clicked()), this, SLOT(activate()));
    toolBar->insertWidget(btnId, button->width(), button, index);
    addContainer(toolBar, btnId);
    connect(toolBar, SIGNAL(destroyed()), this, SLOT(slotDestroyed()));
    button->setFlat(TRUE);
    buttonPtr = button;

    return containerCount() - 1;
  }
  return -1;
}


void CActButton::unplug(QWidget *widget)
{
  if (widget->inherits("KToolBar")) {
    KToolBar *toolBar = (KToolBar *) widget;
    int btnId = findContainer(toolBar);

    if (btnId != -1) {
      toolBar->removeItem(itemId(btnId));
      removeContainer(btnId);
    }
  }
}


int CActButton::widthHint()
{
  if (buttonPtr) return buttonPtr->sizeHint().width(); else return 0;
}


void CActButton::setWidth(int width)
{
  if (buttonPtr) buttonPtr->setFixedWidth(width);
}


//-----------------------
#include "dictactions.moc"
