/***************************************************************************
                          dictconfig.cpp  -  description
                             -------------------
    begin                : . .. 22 20:37:33 ICT 2002
    copyright            : (C) 2002 by Thanomsub Noppaburana
    email                : donga_n@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qdesktopwidget.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qpainter.h>
#include <qtextcodec.h>

#include <kconfig.h>
#include <kcharsets.h>
#include <kglobalsettings.h>
#include <kcolordialog.h>
#include <kfontdialog.h>
#include <kconfig.h>
#include <knumvalidator.h>
#include <knuminput.h>
#include <klineedit.h>
#include <kiconloader.h>
#include <klocale.h>

#include "dictconfig.h"


// ***************************************************************
//              Class CGlobalData Implementation
// ***************************************************************
CGlobalData &CGlobalData::assign(const CGlobalData &gbData)
{
  useCustomColors = gbData.useCustomColors;
  useCustomFonts  = gbData.useCustomFonts;

  // ** Colors section **
  resColors[CNormalText]        = gbData.resColors[CNormalText];
  resColors[CVocab]             = gbData.resColors[CVocab];
  resColors[CMeaning]           = gbData.resColors[CMeaning];
  resColors[CVocabType]         = gbData.resColors[CVocabType];
  resColors[CSpell]             = gbData.resColors[CSpell];
  resColors[CSynonym]           = gbData.resColors[CSynonym];
  resColors[CAntonym]           = gbData.resColors[CAntonym];
  resColors[CBackground]        = gbData.resColors[CBackground];
  resColors[CHeading]           = gbData.resColors[CHeading];
  resColors[CBackgroundHeading] = gbData.resColors[CBackgroundHeading];
  resColors[CFollowVocab]       = gbData.resColors[CFollowVocab];
  for(int i=0; i < colorCount(); i++) resColorNames[i] = gbData.resColorNames[i];

  // ** Fonts section **
  resFonts[FNormalText]         = gbData.resFonts[FNormalText];
  resFonts[FMeaning]            = gbData.resFonts[FMeaning];
  resFonts[FHeading]            = gbData.resFonts[FHeading];
  resFonts[FVocab]              = gbData.resFonts[FVocab];
  resFonts[FSynonym]            = gbData.resFonts[FSynonym];
  resFonts[FAntonym]            = gbData.resFonts[FAntonym];
  resFonts[FVocabType]          = gbData.resFonts[FVocabType];
  resFonts[FSpell]              = gbData.resFonts[FSpell];
  resFonts[FFollowVocab]        = gbData.resFonts[FFollowVocab];
  for(int i=0; i < fontCount(); i++) resFontNames[i] = gbData.resFontNames[i];

  resCharset                    = gbData.resCharset;
  watchClipboard                = gbData.watchClipboard;
  watchInterval                 = gbData.watchInterval;
  saveHistory                   = gbData.saveHistory;
  maxHistoryEntry               = gbData.maxHistoryEntry;
  maxBrowseEntry                = gbData.maxBrowseEntry;
  maxWordListEntry              = gbData.maxWordListEntry;
  showWordList                  = gbData.showWordList;
  searchFromClipboard           = gbData.searchFromClipboard;
  showDockIcon                  = gbData.showDockIcon;

  return *this;
}


CGlobalData &CGlobalData::operator =(const CGlobalData &gbData)
{
  assign(gbData);
  return *this;
}


void CGlobalData::readConfig()
{
  KConfig *config = KGlobal::config();

  // ** General section **
  config->setGroup("General");
    showMainWindow      = config->readBoolEntry("ShowMainWindow", true);
    saveHistory         = config->readBoolEntry("SaveHistory", true);
    maxHistoryEntry     = config->readNumEntry("MaxHistoryEntry", 50);
    maxBrowseEntry      = config->readNumEntry("MaxBrowseEntry", 15);
    maxWordListEntry    = config->readNumEntry("MaxWordListEntry", 20);
    showWordList        = config->readBoolEntry("ShowWordList", true);
    searchFromClipboard = config->readBoolEntry("SearchFromClipboard", false);
    resCharset          = config->readEntry("ResultCharset", "TIS-620");

  // ** SystemTray section **
  config->setGroup("SystemTray");
    watchInterval  = config->readNumEntry("WatchInterval", 1000);
    watchClipboard = config->readBoolEntry("WatchClipboard", false);
    showDockIcon   = config->readBoolEntry("ShowDockIcon", true);

  // ** Colors section **
  config->setGroup("Colors");
    useCustomColors = config->readBoolEntry("UseCustomColor", false);

    QColor sysColor = KGlobalSettings::textColor();
    resColors[CNormalText] = config->readColorEntry("NormalText", &sysColor);
    resColorNames[CNormalText] = i18n("Normal Text");

    resColors[CVocab] = config->readColorEntry("Vocabulary", &sysColor);
    resColorNames[CVocab] = i18n("Vocabulary Text");

    resColors[CMeaning] = config->readColorEntry("Meaning", &sysColor);
    resColorNames[CMeaning] = i18n("Meaning Text");

    resColors[CVocabType] = config->readColorEntry("VocabType", &sysColor);
    resColorNames[CVocabType] = i18n("Vocabulary Type");

    resColors[CSpell] = config->readColorEntry("Spell", &sysColor);
    resColorNames[CSpell] = i18n("Spell Word");

    resColors[CSynonym] = config->readColorEntry("Synonym", &sysColor);
    resColorNames[CSynonym] = i18n("Synonym Text");

    resColors[CAntonym] = config->readColorEntry("Antonym", &sysColor);
    resColorNames[CAntonym] = i18n("Antonym Text");
    
    sysColor = KGlobalSettings::baseColor();
    resColors[CBackground] = config->readColorEntry("Background", &sysColor);
    resColorNames[CBackground] = i18n("Background");

    sysColor = KGlobalSettings::highlightedTextColor();
    resColors[CHeading] = config->readColorEntry("Heading", &sysColor);
    resColorNames[CHeading] = i18n("Header Text");

    sysColor = KGlobalSettings::highlightColor();
    resColors[CBackgroundHeading] = config->readColorEntry("BackgroundHeading", &sysColor);
    resColorNames[CBackgroundHeading] = i18n("Header Background");

    sysColor = KGlobalSettings::linkColor();
    resColors[CFollowVocab] = config->readColorEntry("FollowVocab", &sysColor);
    resColorNames[CFollowVocab] = i18n("Follow Vocabulary");


  // ** Fonts section **
  config->setGroup("Fonts");
    useCustomFonts = config->readBoolEntry("UseCustomFont", false);

    QFont SysFont = KGlobalSettings::generalFont();
    resFonts[FNormalText] = config->readFontEntry("NormalText", &SysFont);
    resFontNames[FNormalText] = i18n("Normal Text");

    resFonts[FMeaning] = config->readFontEntry("Meaning", &SysFont);
    resFontNames[FMeaning] = i18n("Meaning Text");

    SysFont.setPointSize(SysFont.pointSize() + 5);
    resFonts[FHeading] = config->readFontEntry("Heading", &SysFont);
    resFontNames[FHeading] = i18n("Header Text");

    SysFont = KGlobalSettings::generalFont();
    SysFont.setBold(true);
    resFonts[FVocab] = config->readFontEntry("Vocabulary", &SysFont);
    resFontNames[FVocab] = i18n("Vocabulary Text");

    resFonts[FSynonym] = config->readFontEntry("Synonym", &SysFont);
    resFontNames[FSynonym] = i18n("Synonym Text");

    resFonts[FAntonym] = config->readFontEntry("Antonym", &SysFont);
    resFontNames[FAntonym] = i18n("Antonym Text");

    SysFont = KGlobalSettings::generalFont();
    SysFont.setItalic(true);
    resFonts[FVocabType] = config->readFontEntry("VocabType", &SysFont);
    resFontNames[FVocabType] = i18n("Vocabulary Type");

    resFonts[FSpell] = config->readFontEntry("SpellWord", &SysFont);
    resFontNames[FSpell] = i18n("Spell Word");

    SysFont.setBold(true);
    resFonts[FFollowVocab] = config->readFontEntry("FollowVocab", &SysFont);
    resFontNames[FFollowVocab] = i18n("Follow Vocab");


  // ** Geometry section **
  config->setGroup("Geometry");
    QSize defSize(480, 360);
    userSize     = config->readSizeEntry("UserSize", &defSize);
    wordListSize = config->readNumEntry("WordListSize", 148);
    splitterSize = config->readIntListEntry("SplitterSize");

  // ** History section **
  config->setGroup("History");
    wordHistory = config->readListEntry("wordHistory");
}


void CGlobalData::writeConfig()
{
  KConfig *config = KGlobal::config();

  // ** General section **
  config->setGroup("General");
    config->writeEntry("ShowMainWindow", showMainWindow);
    config->writeEntry("SaveHistory", saveHistory);
    config->writeEntry("MaxHistoryEntry", maxHistoryEntry);
    config->writeEntry("MaxBrowseEntry", maxBrowseEntry);
    config->writeEntry("MaxWordListEntry", maxWordListEntry);
    config->writeEntry("ShowWordList", showWordList);
    config->writeEntry("SearchFromClipboard", searchFromClipboard);
    config->writeEntry("ResultCharset", resCharset);

  // ** SystemTray section **
  config->setGroup("SystemTray");
    config->writeEntry("WatchInterval", watchInterval);
    config->writeEntry("WatchClipboard", watchClipboard);
    config->writeEntry("ShowDockIcon", showDockIcon);

  // ** Colors section **
  config->setGroup("Colors");
    config->writeEntry("UseCustomColor", useCustomColors);
    config->writeEntry("BackgroundHeading", resColors[CBackgroundHeading]);
    config->writeEntry("Background", resColors[CBackground]);
    config->writeEntry("Heading", resColors[CHeading]);
    config->writeEntry("NormalText", resColors[CNormalText]);
    config->writeEntry("Vocabulary", resColors[CVocab]);
    config->writeEntry("VocabType", resColors[CVocabType]);
    config->writeEntry("SpellWord", resColors[CSpell]);
    config->writeEntry("Meaning", resColors[CMeaning]);
    config->writeEntry("Synonym", resColors[CSynonym]);
    config->writeEntry("Antonym", resColors[CAntonym]);
    config->writeEntry("FollowVocab", resColors[CFollowVocab]);

  // ** Fonts section **
  config->setGroup("Fonts");
    config->writeEntry("UseCustomFont", useCustomFonts);
    config->writeEntry("Heading", resFonts[FHeading]);
    config->writeEntry("NormalText", resFonts[FNormalText]);
    config->writeEntry("Vocabulary", resFonts[FVocab]);
    config->writeEntry("VocabType", resFonts[FVocabType]);
    config->writeEntry("SpellWord", resFonts[FSpell]);
    config->writeEntry("Meaning", resFonts[FMeaning]);
    config->writeEntry("Synonym", resFonts[FSynonym]);
    config->writeEntry("Antonym", resFonts[FAntonym]);
    config->writeEntry("FollowVocab", resFonts[FFollowVocab]);
    
  // ** Geometry section **
  config->setGroup("Geometry");
    config->writeEntry("UserSize", userSize);
    config->writeEntry("WordListSize", wordListSize);
    config->writeEntry("SplitterSize", splitterSize);

  // ** History section **
  config->setGroup("History");
    QStringList copy;
    if (saveHistory) copy = wordHistory;
    config->writeEntry("wordHistory", copy);
}


void CGlobalData::setDefaultValue(ConfigGroup grp)
{
  QColor SysColor;
  QFont  SysFont;
  
  switch(grp) {
    case cfgFontcolor:
      useCustomColors = false;
      useCustomFonts  = false;
      
      // ** Colors section **
      SysColor = KGlobalSettings::textColor();
      resColors[CNormalText] = SysColor;
      resColors[CVocab]      = SysColor;
      resColors[CMeaning]    = SysColor;
      resColors[CVocabType]  = SysColor;
      resColors[CSpell]      = SysColor;
      resColors[CSynonym]    = SysColor;
      resColors[CAntonym]    = SysColor;

      SysColor = KGlobalSettings::baseColor();
      resColors[CBackground] = SysColor;

      SysColor = KGlobalSettings::highlightedTextColor();
      resColors[CHeading] = SysColor;

      SysColor = KGlobalSettings::highlightColor();
      resColors[CBackgroundHeading] = SysColor;

      SysColor = KGlobalSettings::linkColor();
      resColors[CFollowVocab] = SysColor;

      // ** Fonts section **
      SysFont = KGlobalSettings::generalFont();
      resFonts[FNormalText] = SysFont;
      resFonts[FMeaning]    = SysFont;

      SysFont.setPointSize(SysFont.pointSize() + 5);
      resFonts[FHeading] = SysFont;

      SysFont = KGlobalSettings::generalFont();
      SysFont.setBold(true);
      resFonts[FVocab]   = SysFont;
      resFonts[FSynonym] = SysFont;
      resFonts[FAntonym] = SysFont;

      SysFont = KGlobalSettings::generalFont();
      SysFont.setItalic(true);
      resFonts[FVocabType] = SysFont;
      resFonts[FSpell]     = SysFont;
      
      SysFont.setBold(true);
      resFonts[FFollowVocab] = SysFont;
    break;
    
    case cfgAppearance:
      resCharset = "TIS-620";
    break;
    
    case cfgTransbehavior:
      watchClipboard = false;
      watchInterval  = 1000;
      resCharset     = "TIS-620";
    break;
    
    case cfgDisplay:
    break;
    
    case cfgMisc:
      saveHistory         = true;
      maxHistoryEntry     = 50;
      maxBrowseEntry      = 15;
      maxWordListEntry    = 20;
      showWordList        = true;
      searchFromClipboard = false;
      showDockIcon        = true;
    break;
  }
}


// ***************************************************************
//       Class COptionDialog::CFontListItem Implementation
// ***************************************************************

COptionDialog::CFontListItem::CFontListItem(const QString &name, const QFont &font)
  : QListBoxText(name), selfFont(font)
{
  fontInfo = QString("[%1 %2]").arg(selfFont.family()).arg(selfFont.pointSize());
}


COptionDialog::CFontListItem::~CFontListItem()
{}


void COptionDialog::CFontListItem::setFont(const QFont &font)
{
  selfFont = font;
  fontInfo = QString("[%1 %2]").arg(selfFont.family()).arg(selfFont.pointSize());
}


void COptionDialog::CFontListItem::paint(QPainter *p)
{
  QFont fnt = p->font();
  //fnt.setWeight(QFont::Bold);
  p->setFont(selfFont);
  //resize(p->window().width(), p->fontMetrics().height());
  int fontInfoWidth = p->fontMetrics().width(fontInfo);
  int h = p->fontMetrics().ascent() + p->fontMetrics().leading() / 2;
  p->drawText(2, h, fontInfo);
  //fnt.setWeight(QFont::Normal);
  p->setFont(fnt);
  p->drawText(5 + fontInfoWidth, h, text());
}


int COptionDialog::CFontListItem::width(const QListBox *lb) const
{
  return (lb->fontMetrics().width(fontInfo) + lb->fontMetrics().width(text()) + 20);
}


// ***************************************************************
//       Class COptionDialog::CFontListItem Implementation
// ***************************************************************


COptionDialog::CColorListItem::CColorListItem(const QString &name, const QColor &color)
  : QListBoxText(name), selfColor(color)
{
}


COptionDialog::CColorListItem::~CColorListItem()
{
}


void COptionDialog::CColorListItem::paint(QPainter *p)
{
  QFontMetrics fm = p->fontMetrics();
  int h = fm.height();

  p->drawText( 30+3*2, fm.ascent() + fm.leading()/2, text() );

  p->setPen(Qt::black);
  p->drawRect(3, 1, 30, h-1);
  p->fillRect(4, 2, 28, h-3, selfColor);
}


int COptionDialog::CColorListItem::height(const QListBox *lb) const
{
  return (lb->fontMetrics().lineSpacing()+1);
}


int COptionDialog::CColorListItem::width(const QListBox *lb) const
{
  return (30 + lb->fontMetrics().width(text()) + 6);
}


// ***************************************************************
//                   Class COptionDialog Implementation
// ***************************************************************

COptionDialog::COptionDialog(QWidget *parent, char *name, bool modal)
  : KDialogBase(IconList, i18n("Preferences"),
       Help|Default|Apply|Ok|Cancel, Ok, parent, name, modal, true),
    fontColorPage(0), appPage(0), transPage(0), miscPage(0)
{
  defVal.assign(*global);
  setHelp("kdictthai/index.html", QString::null);
  setupFontAndColorPage();
  setupResultAppearancePage();
  setupTransBehaviorPage();
  setupMiscPage();
  resize(480, 320);
}


COptionDialog::~COptionDialog()
{
}


void COptionDialog::setupFontAndColorPage()
{
  if (! fontColorPage) {
    fontColorPage = addPage(i18n("Fonts & Colors"), i18n("Customize Fonts and Colors"), BarIcon("fonts", KIcon::SizeMedium));

    QGridLayout *topL=new QGridLayout(fontColorPage, 8,3, 5,5);

    // ** Color list **
    colorCB = new QCheckBox(i18n("&Use custom colors"), fontColorPage);
    topL->addWidget(colorCB, 0, 0);
    connect(colorCB, SIGNAL(toggled(bool)), this, SLOT(slotColorCheckBoxToggled(bool)));

    colorList = new QListBox(fontColorPage);
    topL->addMultiCellWidget(colorList,1,3,0,1);
    connect(colorList, SIGNAL(selected(QListBoxItem*)),SLOT(slotColorItemSelected(QListBoxItem*)));

    // ** Font list **
    fontCB = new QCheckBox(i18n("Use custom &fonts"), fontColorPage);
    topL->addWidget(fontCB,4,0);
    connect(fontCB, SIGNAL(toggled(bool)), SLOT(slotFontCheckBoxToggled(bool)));

    fontList = new QListBox(fontColorPage);
    topL->addMultiCellWidget(fontList,5,7,0,1);
    connect(fontList, SIGNAL(selected(QListBoxItem*)),SLOT(slotFontItemSelected(QListBoxItem*)));
    
    // ** init **
    for(int i=0; i< defVal.colorCount(); i++)
       colorList->insertItem(new CColorListItem(defVal.colorName(i), defVal.color(i)));

    for(int i=0; i < defVal.fontCount(); i++)
      fontList->insertItem( new CFontListItem(defVal.fontName(i), defVal.font(i)) );
  }
  colorCB->setChecked(defVal.useCustomColors);  slotColorCheckBoxToggled(defVal.useCustomColors);
  fontCB->setChecked(defVal.useCustomFonts);    slotFontCheckBoxToggled(defVal.useCustomFonts);
}


void COptionDialog::setupResultAppearancePage()
{
  QLabel *label;

  if (! appPage) {
    appPage = addPage(i18n("Appearance"), i18n("Customize visual appearance"), BarIcon("appearance", KIcon::SizeMedium));
    QVBoxLayout *vbox = new QVBoxLayout(appPage, 5, 0);

    QGroupBox *group = new QGroupBox(i18n("Result view"), appPage);
  
    QGridLayout *grid = new QGridLayout(group, 4, 2, 8, 5);
    grid->addRowSpacing(0, fontMetrics().lineSpacing() - 4);

    label = new QLabel(group, "label");
    label->setText(i18n( "Select the charset to use when display Query results. If you are not sure select ISO-8859-1 (latin1)."));
    label->setTextFormat(QLabel::AutoText);
    label->setAlignment(int(QLabel::WordBreak | QLabel::AlignVCenter | QLabel::AlignLeft));
    grid->addMultiCellWidget(label, 1, 1, 0, 1);

    charsetLabel = new QLabel(group, "charsetLabel");
    charsetLabel->setText(i18n("&Charset"));
  
    charsetCombo = new QComboBox(false, group, "charsetCombo");
    charsetCombo->setMinimumSize(QSize(150, 0));
  
    QTextCodec *codec;
    QStringList codecList;
    int n=0;
    do {
      codec = QTextCodec::codecForIndex(n++);
      if (codec!=NULL) codecList.append(codec->mimeName());
    } while (codec!=NULL);
  
    codecList.sort();
    charsetCombo->insertStringList(codecList);
    charsetLabel->setBuddy(charsetCombo);
  
    grid->addWidget(charsetLabel, 2, 0);
    grid->addWidget(charsetCombo, 2, 1);

    grid->setColStretch(1, 1);

    vbox->addWidget(group, 0);
    vbox->addStretch(2);
  }

  charsetCombo->setCurrentText(defVal.resCharset);
}


void COptionDialog::setupTransBehaviorPage()
{
  QLabel *label;

  if (! transPage) {
    transPage = addPage(i18n("Translation"), i18n("Customize Translation behavior"), BarIcon("locale", KIcon::SizeMedium));

    QVBoxLayout *vbox = new QVBoxLayout(transPage, 5, 0);
    watchGroup = new QGroupBox(i18n("Watch clipboard"), transPage);

    watchClipboard = new QCheckBox(i18n("Watch word on clipboard"), transPage);
    connect(watchClipboard, SIGNAL(toggled(bool)), this, SLOT(slotWatchClipboardToggled(bool)));
    vbox->addWidget(watchClipboard, 0);
  
    QGridLayout *grid = new QGridLayout(watchGroup, 3, 2, 8, 5);
    grid->addRowSpacing(0, fontMetrics().lineSpacing() - 4);
  
    watchInterval  = new KIntSpinBox(50, 5000, 50, 50, 10, watchGroup);
    label = new QLabel(watchInterval, i18n("Watch interval (msec):"), watchGroup);
    label->setBuddy(watchInterval);
    grid->addWidget(label, 2, 0);
    grid->addWidget(watchInterval, 2, 1);

    grid->setColStretch(1, 1);

    vbox->addWidget(watchGroup, 0);
    vbox->addStretch(1);
    
    ////////////////////////
    
    QGroupBox *resGroup = new QGroupBox(i18n("Result view"), transPage);

    grid = new QGridLayout(resGroup, 4, 2, 8, 5);
    grid->addRowSpacing(0, fontMetrics().lineSpacing() - 4);

    label = new QLabel(resGroup, "label");
    label->setText(i18n( "Select the charset to use when display Query results. If you are not sure select ISO-8859-1 (latin1)."));
    label->setTextFormat(QLabel::AutoText);
    label->setAlignment(int(QLabel::WordBreak | QLabel::AlignVCenter | QLabel::AlignLeft));
    grid->addMultiCellWidget(label, 1, 1, 0, 1);

    charsetLabel = new QLabel(resGroup, "charsetLabel");
    charsetLabel->setText(i18n("&Charset"));

    charsetCombo = new QComboBox(false, resGroup, "charsetCombo");
    charsetCombo->setMinimumSize(QSize(150, 0));

    QTextCodec *codec;
    QStringList codecList;
    int n=0;
    do {
      codec = QTextCodec::codecForIndex(n++);
      if (codec!=NULL) codecList.append(codec->mimeName());
    } while (codec!=NULL);

    codecList.sort();
    charsetCombo->insertStringList(codecList);
    charsetLabel->setBuddy(charsetCombo);

    grid->addWidget(charsetLabel, 2, 0);
    grid->addWidget(charsetCombo, 2, 1);

    grid->setColStretch(1, 1);

    vbox->addWidget(resGroup, 0);
    
    vbox->addStretch(2);
    
    /////////////

  }
  watchInterval->setValue(defVal.watchInterval);
  watchClipboard->setChecked(defVal.watchClipboard);
  watchGroup->setEnabled(defVal.watchClipboard);
  charsetCombo->setCurrentText(defVal.resCharset);
}


void COptionDialog::setupMiscPage()
{
  QLabel *label;

  if (! miscPage) {
    miscPage = addPage(i18n("Miscellaneous"), i18n("Various properties"), BarIcon("misc", KIcon::SizeMedium));
    QVBoxLayout *vbox = new QVBoxLayout(miscPage, 5, 0);

    QGroupBox *group = new QGroupBox(i18n("History"), miscPage);

    QGridLayout *grid = new QGridLayout(group, 4, 2, 8, 5);
    grid->addRowSpacing(0, fontMetrics().lineSpacing() - 4);

    valMaxbrowse = new KIntSpinBox(1, 100, 1, 1, 10, group);
    label = new QLabel(valMaxbrowse, i18n("Browse &Entrys:"), group);
    grid->addWidget(label, 1, 0);
    grid->addWidget(valMaxbrowse, 1, 1);

    valMaxhistory = new KIntSpinBox(10, 5000, 10, 10, 10, group);
    label = new QLabel(valMaxhistory, i18n("Hi&story Entrys:"), group);
    grid->addWidget(label, 2, 0);
    grid->addWidget(valMaxhistory, 2, 1);

    valMaxwordlist = new KIntSpinBox(10, 5000, 10, 10, 10, group);
    label = new QLabel(valMaxwordlist, i18n("Word&List Entrys:"), group);
    grid->addWidget(label, 3, 0);
    grid->addWidget(valMaxwordlist, 3, 1);
  
    grid->setColStretch(1, 1);

    vbox->addWidget(group, 0);

    group = new QGroupBox(i18n("Other"), miscPage);
    QVBoxLayout *vbox2 = new QVBoxLayout(group, 8, 5);
    vbox2->addSpacing(fontMetrics().lineSpacing() - 4);

    valSavehistory = new QCheckBox(i18n("Sa&ve history on exit"), group);
    vbox2->addWidget(valSavehistory, 0);

    valClipboard = new QCheckBox(i18n("S&earch selected text on start"), group);
    vbox2->addWidget(valClipboard, 1);

    valShowWordList = new QCheckBox(i18n("Show &relate word lists"), group);
    vbox2->addWidget(valShowWordList, 2);
  
    vbox->addWidget(group, 0);
    vbox->addStretch(2);

    setHelp("preferences");
  }
  
  valMaxbrowse->setValue(defVal.maxBrowseEntry);
  valMaxhistory->setValue(defVal.maxHistoryEntry);
  valMaxwordlist->setValue(defVal.maxWordListEntry);
  valSavehistory->setChecked(defVal.saveHistory);
  valClipboard->setChecked(defVal.searchFromClipboard);
  valShowWordList->setChecked(defVal.showWordList);
}


// ********* COptionDialog slot implement *********

void COptionDialog::slotDefault()
{
  defVal.setDefaultValue((ConfigGroup) activePageIndex());
  
  switch (activePageIndex()) {
    case page_fontcolor:
      setupFontAndColorPage();
    break;
    
    case page_appearance:
      setupResultAppearancePage();
    break;

    case page_transbehavior:
      setupTransBehaviorPage();
    break;

    case page_display:
      // setupDisplay();
    break;

    case page_misc:
      setupMiscPage();
    break;
  }
}

void COptionDialog::slotOk()
{
  slotApply();
  accept();
  KDialogBase::slotOk();
}

void COptionDialog::slotApply()
{
  //defVal->showDockIcon        = valShowDockIcon->isChecked();
  defVal.showWordList        = valShowWordList->isChecked();
  defVal.searchFromClipboard = valClipboard->isChecked();
  defVal.saveHistory         = valSavehistory->isChecked();
  defVal.maxHistoryEntry     = valMaxhistory->value();
  defVal.maxBrowseEntry      = valMaxbrowse->value();
  defVal.maxWordListEntry    = valMaxwordlist->value();
  defVal.resCharset          = charsetCombo->currentText();
  defVal.watchInterval       = watchInterval->value();
  defVal.watchClipboard      = watchClipboard->isChecked();
  
  defVal.useCustomColors = colorCB->isChecked();
  for(int i=0; i < defVal.colorCount(); i++)
    defVal.resColors[i] = (static_cast<CColorListItem*>(colorList->item(i)))->color();

  defVal.useCustomFonts = fontCB->isChecked();
  for(int i=0; i < defVal.fontCount(); i++)
    defVal.resFonts[i] = (static_cast<CFontListItem*>(fontList->item(i)))->font();

  global->assign(defVal);
  emit(optionChanged());  
}


void COptionDialog::slotColorCheckBoxToggled(bool checked)
{
  colorList->setEnabled(checked);
  if (checked) colorList->setFocus();
}


void COptionDialog::slotColorItemSelected(QListBoxItem *item)
{
  if (item) {
    CColorListItem *colorItem = static_cast<CColorListItem*>(item);
    QColor col = colorItem->color();
    int result = KColorDialog::getColor(col,this);
    if (result == KColorDialog::Accepted)
    {
      colorItem->setColor(col);
      colorList->triggerUpdate(false);
    }
  }
}


void COptionDialog::slotFontCheckBoxToggled(bool checked)
{
  fontList->setEnabled(checked);
  if (checked) fontList->setFocus();
}


void COptionDialog::slotWatchClipboardToggled(bool checked)
{
  watchGroup->setEnabled(checked);
  if (checked) watchInterval->setFocus();
}


void COptionDialog::slotFontItemSelected(QListBoxItem *item)
{
  if (item) {
    CFontListItem *fontItem = static_cast<CFontListItem*>(item);
    QFont font = fontItem->font();
    int result = KFontDialog::getFont(font, false, this);
    if (result == KFontDialog::Accepted)
    {
      fontItem->setFont(font);
      fontList->triggerUpdate(false);
    }
  }
}


//--------------------------
#include "dictconfig.moc"
