/***************************************************************************
                          dictconfig.h  -  description
                             -------------------
    begin                : . .. 22 20:37:33 ICT 2002
    copyright            : (C) 2002 by Thanomsub Noppaburana
    email                : donga_n@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef __DICTCONFIG_H__
#define __DICTCONFIG_H__

#include <kdialogbase.h>


class QListbox;
class QCheckBox;
class QComboBox;
class QGroupBox;
class KIntSpinBox;


// *****************************

const int FONTCOUNT  =  9;
const int COLORCOUNT = 11;

enum FontType  { FHeading=0, FNormalText, FVocab, FVocabType, FSpell, FMeaning, FSynonym, FAntonym, FFollowVocab };
enum ColorType { CHeading=0, CBackgroundHeading, CBackground, CNormalText, CVocab, CVocabType, CSpell, CMeaning, CSynonym, CAntonym, CFollowVocab };
enum ConfigGroup { cfgFontcolor = 0, cfgAppearance, cfgTransbehavior, cfgDisplay, cfgMisc };

    
class CGlobalData
{
  public:

    CGlobalData &operator =(const CGlobalData &);
    CGlobalData &assign(const CGlobalData &);
    
    void readConfig();
    void writeConfig();

    int fontCount() { return FONTCOUNT; };
    const QFont &font(int i) { return resFonts[i]; };
    const QString &fontName(int i) { return resFontNames[i]; };

    int colorCount() { return COLORCOUNT; };
    const QColor &color(int i) { return resColors[i]; };
    const QString &colorName(int i) { return resColorNames[i]; };

    void setDefaultValue(ConfigGroup);

    // ** General section ** //
    QString resCharset;
    bool showMainWindow;

    // ** Dock Icon** //
    bool showDockIcon;
    bool watchClipboard;
    int  watchInterval;

    // ** Word Listbox ** //
    unsigned int maxWordListEntry;
    bool showWordList;
    bool searchFromClipboard;

    // ** Colors section ** //
    bool useCustomColors;

    // ** Fonts section ** //
    bool useCustomFonts;

    // ** Geometry section ** //
    QPoint           topLeft;
    QSize            userSize;
    int              wordListSize;
    QValueList<int>  splitterSize;

    // ** History section ** //
    bool saveHistory;
    QStringList  wordHistory;
    unsigned int maxHistoryEntry;
    unsigned int maxBrowseEntry;


    QFont   resFonts[FONTCOUNT];
    QString resFontNames[FONTCOUNT];
    QColor  resColors[COLORCOUNT];
    QString resColorNames[COLORCOUNT];
};


extern CGlobalData *global;


// ***************************** //

class COptionDialog : public KDialogBase
{
  Q_OBJECT
  // ================== //
  class CFontListItem : public QListBoxText {
    public:
      CFontListItem(const QString &name, const QFont &);
      ~CFontListItem();
      const QFont& font() { return selfFont; }
      void setFont(const QFont &);

    protected:
      virtual void paint(QPainter *);
      virtual int width(const QListBox *) const;

    private:
      QFont   selfFont;
      QString fontInfo;
  };
  // ================== //
  class CColorListItem : public QListBoxText {

    public:
      CColorListItem(const QString &name, const QColor &color=Qt::black);
      ~CColorListItem();
      const QColor& color() { return selfColor; }
      void  setColor(const QColor &color) { selfColor = color; }

    protected:
      virtual void paint(QPainter *);
      virtual int height(const QListBox *) const;
      virtual int width(const QListBox *) const;

    private:
      QColor selfColor;
  };
  // ================== //
  
  public:
    enum Page { page_fontcolor = 0, page_appearance, page_transbehavior, page_display, page_misc };

    COptionDialog(QWidget *parent = 0, char *name = 0, bool modal = FALSE);
    ~COptionDialog();

  signals:
    void optionChanged();

  public slots:
    void slotDefault();
    void slotOk();
    void slotApply();

    void slotColorCheckBoxToggled(bool);
    void slotColorItemSelected(QListBoxItem *);

    void slotFontCheckBoxToggled(bool);
    void slotFontItemSelected(QListBoxItem *);

    void slotWatchClipboardToggled(bool);

  private:
    void setupFontAndColorPage();
    void setupResultAppearancePage();
    void setupTransBehaviorPage();
    void setupMiscPage();

    // ** Temporary Global **
    CGlobalData defVal;

    // ** Font & Color Page **
    QFrame    *fontColorPage;
    QCheckBox *colorCB;
    QListBox  *colorList;
    QListBox  *fontList;
    QCheckBox *fontCB;

    // ** Result Appearance Page **
    QFrame    *appPage;
    QLabel    *charsetLabel;
    QComboBox *charsetCombo;

    // ** Translation behavior **
    QFrame      *transPage;
    QGroupBox   *watchGroup;
    QCheckBox   *watchClipboard;
    KIntSpinBox *watchInterval;

    // ** Miscellaneous Page **
    QFrame      *miscPage;
    KIntSpinBox *valMaxwordlist;
    KIntSpinBox *valMaxhistory;
    KIntSpinBox *valMaxbrowse;
    QCheckBox   *valSavehistory;
    QCheckBox   *valClipboard;
    QCheckBox   *valShowWordList;
};


#endif
