/***************************************************************************
                          dictdb.cpp  -  description
                             -------------------
    begin                : . .. 21 2002
    copyright            : (C) 2002 by Thanomsub Noppaburana
    email                : donga_n@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 
#include <db_cxx.h>

#include <iomanip>
#include <string>
using namespace std;

#include "dictdb.h"
#include "dictapplication.h"



// ***************************************************************
//                   Class CDictData Implementation
// ***************************************************************

CDictData::CDictData(const char *dbFile)
  : dbPtr(0), dbCursorPtr(0), currentKey(0), currentData(0), searchMode(smMatchRange)
{
  dictResult = new CDictRecord();
  
  dbPtr = new Db(NULL, 0);
  dbPtr->set_error_stream(&cerr);
  dbPtr->set_errpfx(PROGNAME);
  dbPtr->set_pagesize(1024);        // 1K page sizes.
  dbPtr->set_flags(DB_RECNUM);      // Record numbers.
  //dbPtr->open(dbFile, NULL, DB_BTREE, DB_CREATE, 0);
  dbPtr->open(0, dbFile, NULL, DB_BTREE, DB_CREATE,0);
  dbPtr->cursor(NULL, &dbCursorPtr, 0);
}


CDictData::~CDictData()
{
  if (dbCursorPtr!=0) dbCursorPtr->close();
  dbPtr->close(0);
  delete dbPtr;
}


CDictRecord *CDictData::dictRecord()
{
  return dictResult;
}


int CDictData::recordsCount()
{
  DB_BTREE_STAT *statPtr;
  dbPtr->stat(&statPtr, DB_RECORDCOUNT);
  int result = statPtr->bt_ndata;
  free(statPtr); // must use free, not delete, this struct is allocated by C.
  return result;
}


void CDictData::setMode(int mode)
{
  searchMode = mode;
}


bool CDictData::searchDict(char *keyword)
{
  if (!keyword) return srcNotfound;

  int ret;
  
  if (!currentKey && !currentData) {
    currentKey  = new Dbt(keyword, strlen(keyword));
    currentData = new Dbt();
  } else {
    currentKey->set_data(keyword);
    currentKey->set_size(strlen(keyword));
  }
  

  if ((ret = dbCursorPtr->get(currentKey, currentData, DB_SET | DB_SET_RANGE)) != 0) {
    dbPtr->err(ret, "CDictData->searchDict");
    throw DbException(ret);
  }
  dictResult->word    = (char*) currentKey->get_data();
  dictResult->meaning = (char*) currentData->get_data();

  if (dictResult->word != "") return srcFound; else return srcNotfound;
}


bool CDictData::searchDict(const char *keyword)
{
  return searchDict((char *)keyword);
}


CDictRecord *CDictData::nextRecord()
{
  if (!currentKey && !currentData) return NULL;

  int ret;

  if ((ret = dbCursorPtr->get(currentKey, currentData, DB_NEXT)) != 0) {
    dbPtr->err(ret, "CDictData->nextRecord");
    throw DbException(ret);
  }
  dictResult->word    = (char*) currentKey->get_data();
  dictResult->meaning = (char*) currentData->get_data();

  //if (dictResult->word != "") return srcFound; else return srcNotfound;
  return NULL;
}


char *CDictData::Keyword() const
{
  return (char*)dictResult->word.c_str();
}


char *CDictData::Meaning() const
{
  return (char*)dictResult->meaning.c_str();  
}



// ---------------------
//#include "dictdb.moc"
