/***************************************************************************
                          dictengine.cpp  -  description
                             -------------------
    begin                : . .. 22 2002
    copyright            : (C) 2002 by Thanomsub Noppaburana
    email                : donga_n@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qobject.h>
#include <qtextcodec.h>
#include <qregexp.h>

#include <kstandarddirs.h>
#include <kdebug.h>

#include "dictconfig.h"
#include "dictdb.h"
#include "dictapplication.h"
#include "dictengine.h"


// ***************************************************************
//                 Class CDictEngine Implementation
// ***************************************************************

CDictEngine::CDictEngine() : dictRec(NULL), maxRelateWord(global->maxWordListEntry)
{
  KStandardDirs *dir = KGlobal::dirs();
  QStringList dbPathList = dir->findAllResources("appdata", DICTDATABASE, true);
  kdDebug(5004) << "Load DB : " << dbPathList[0] << endl;
  dictDB = new CDictData(dbPathList[0]);
  kdDebug(5004) << "Load DB Success..." << endl;
}


CDictEngine::~CDictEngine()
{
}


void CDictEngine::setMaxRelateWords(const int max)
{
  maxRelateWord = max;
}


int CDictEngine::recordsCount()
{
  return dictDB->recordsCount();
}


QString CDictEngine::queryText()
{
  return _queryText;
}


QString CDictEngine::keyword()
{
  if (dictRec)
    return convertCodec(dictRec->word.c_str());
  else
    return "";
}


QString CDictEngine::meaning()
{
  QString res(dictRec->meaning.c_str());
  
  if (dictRec) {
    res = convertCodec(res);
    res = generateHTML(res);
    return res;
  } else
    return "";
}


bool CDictEngine::searchWithNoSignal(const char *key)
{ 
  return searchWithNoSignal(QString(key).lower());
}


bool CDictEngine::searchWithNoSignal(const QString &key)
{
  bool result = dictDB->searchDict(key.lower());
  dictRec = dictDB->dictRecord();
  _queryText = keyword();
  emit(resultReady("<span class=\"vocab\">" + _queryText + "</span> " + meaning()));
  return result;
}


bool CDictEngine::searchKeyword(const char *key)
{
  return searchWithNoSignal(QString(key));
}


bool CDictEngine::searchKeyword(const QString &key)
{
  bool result = searchWithNoSignal(key);
  emit(searchComplete());
  return result;  
}


void CDictEngine::goNextRecord()
{
  dictDB->nextRecord();
}


QStringList &CDictEngine::getRelateWordList()
{
  QStringList *result = new QStringList();
  for (int i=0; i < maxRelateWord; i++) {
    result->append(keyword());
    goNextRecord();
  }
  return *result;
}


QString CDictEngine::convertCodec(const QString &str)
{
  QTextCodec *codec = QTextCodec::codecForName(global->resCharset);
  QString result = codec->toUnicode(str);
  return result;
}


QString CDictEngine::pharseString(QString &str)
{
  return "";
}


QString CDictEngine::generateHTML(QString &str)
{
  QString result(str), tempStr, tmp, lineStr;
  int idx;
  bool firstItem  = true;

  QRegExp rxp("", true, false);
  QRegExp rxp2("", false, false);

  rxp.setPattern(convertCodec("ͧ[a-zA-Z]"));    while( (idx = result.find(rxp)) > -1) result.replace(idx, 3, convertCodec("ͧ "));
  rxp.setPattern(convertCodec("[a-zA-Z]"));      while( (idx = result.find(rxp)) > -1) result.replace(idx, 2, convertCodec(" "));
  rxp.setPattern(convertCodec("[a-zA-Z]"));      while( (idx = result.find(rxp)) > -1) result.replace(idx+1, 2, convertCodec(" "));
  rxp.setPattern(convertCodec("Ѻ[a-zA-Z]"));    while( (idx = result.find(rxp)) > -1) result.replace(idx, 3, convertCodec("Ѻ "));
  rxp.setPattern(convertCodec("[a-zA-Z]Сͺ")); while( (idx = result.find(rxp)) > -1) result.replace(idx+1, 2, convertCodec(" "));
  rxp.setPattern("-ad[j|v]\\.");                  while( (idx = result.find(rxp)) > -1) result.replace(idx, 3, "ad");
  rxp.setPattern("-[p|P]");                       while( (idx = result.find(rxp)) > -1) result.replace(idx, 2, "p");
  rxp.setPattern("- [p|P]");                      while( (idx = result.find(rxp)) > -1) result.replace(idx, 3, "p");
  rxp.setPattern("-v\\.");                        while( (idx = result.find(rxp)) > -1) result.replace(idx, 3, "v.");
  rxp.setPattern("\\.n\\.");                      while( (idx = result.find(rxp)) > -1) result.replace(idx, 3, "n.");
  rxp.setPattern("###");                          while( (idx = result.find(rxp)) > -1) result.replace(idx, 3, "");
  rxp.setPattern("\\.;");                         while( (idx = result.find(rxp)) > -1) result.replace(idx, 2, ".");
  rxp.setPattern(convertCodec(",[a-zA-Z-]"));   while( (idx = result.find(rxp)) > -1) result.replace(idx, 1, ", ");
  rxp.setPattern(convertCodec("[-]\\."));       while( (idx = result.find(rxp)) > -1) result.replace(idx+1, 1, "");
  
  rxp2.setPattern("prep\\.|pron\\.|conj\\.|abbr\\.|Pref\\.|Phr\\.|pro\\.|adv\\.|adj\\.|pre\\.|con\\.|^in\\.|pt\\.|pp\\.|fr\\.|pl\\.|SW\\.|vi\\.|vt\\.|C\\.|S\\.|A\\.|F\\.|^n\\.|^p\\.|^v\\.");
  rxp.setPattern("S\\.|A\\.|SW\\.");
  result = result.simplifyWhiteSpace();

  enum { lNone=0, lType, lMeaning, lAnSynEx  };
  int lastSect = lNone;
    
  bool isAntSynEx = false;
  bool isMeaning  = false;
  bool isType     = false;
  
    idx = 0; tmp = result.section(' ', idx, idx);
    if (tmp.isEmpty()) tempStr = "<br>" + result;

    isAntSynEx = (tmp.find(rxp) != -1);

    while (!tmp.isEmpty()) {
      if (firstItem) {
        if (tmp[0] == '(') {
          tempStr += "<span class=\"spell\">";
          tempStr += (tmp + ((tmp.find(')') == -1) ? " " : "</span>") );
          while (tmp.find(')') == -1) {
            idx++; tmp = result.section(' ', idx, idx);
            tempStr += (tmp + ((tmp.find(')') == -1) ? " " : "</span>") );
          }
          tempStr += "<div class=\"definition\">";
        }
        else {
          tempStr += "<div class=\"definition\">";
          if (tmp.find(rxp2) != -1)
            tempStr += ("<span class=\"vocabtype\">" + tmp + "</span> "); // br first
          else
            tempStr += ("<span class=\"meaning\">" + tmp + "</span> ");   // br first
        }
        firstItem = false;
      }
      else {
        if (!isAntSynEx && (tmp.find(rxp2) != -1)) {
          isAntSynEx = (tmp.find(rxp) != -1);
          if (!lineStr.isEmpty()) {
            tempStr += (lineStr + ((isAntSynEx)||(lastSect!=lType) ? "<br>":" ") );
          }
          lineStr  = "<span class=\"vocabtype\">" + tmp + "</span> ";
          lastSect = lType;
        }
        else {
          if (!isMeaning) {
            isType = (tmp.find(rxp2) != -1);
            if (!isType)
              lineStr += "<span class=\"meaning\">";
            else
              lineStr += "<span class=\"vocabtype\">";
            
            isMeaning = true;
          }
          lineStr += (tmp + " ");
          if (isAntSynEx) lastSect = lType; else lastSect = lNone;
        }
      }
      idx++; tmp = result.section(' ', idx, idx);
      if (tmp.find(rxp) != -1) isAntSynEx = false;
      if (tmp.find(rxp2) != -1) {
        if (isMeaning) {
          lineStr += "</span>";
          isMeaning = false;
        }
      }
    }
    tempStr += lineStr;
    tempStr += "</div>";
    
  //rxp.setPattern(""); while( (idx = result.find(rxp)) > -1) result.replace(idx+1, 2, " ");

  return tempStr.simplifyWhiteSpace();
}


//--------------------------
#include "dictengine.moc"


/*

QString CDictEngine::optimizeString(QString &str)
{
  QString result(str), tempStr, tmp, lineStr;
  int idx;
  bool firstItem  = true;

  QRegExp rxp("", true, false);
  QRegExp rxp2("", false, false);

  rxp.setPattern("[a-zA-Z]"); 	while( (idx = result.find(rxp)) > -1) result.replace(idx, 2, " ");
  rxp.setPattern("[a-zA-Z]"); 	while( (idx = result.find(rxp)) > -1) result.replace(idx+1, 2, " ");
  rxp.setPattern("Ѻ[a-zA-Z]"); 	while( (idx = result.find(rxp)) > -1) result.replace(idx, 3, "Ѻ ");
  rxp.setPattern("[a-zA-Z]Сͺ");	while( (idx = result.find(rxp)) > -1) result.replace(idx+1, 2, " ");
  rxp.setPattern("-[p|P]"); while( (idx = result.find(rxp)) > -1) result.replace(idx, 2, "p");
  rxp.setPattern("- [p|P]"); while( (idx = result.find(rxp)) > -1) result.replace(idx, 3, "p");
  rxp.setPattern("-v\\."); while( (idx = result.find(rxp)) > -1) result.replace(idx, 3, "v.");
  rxp.setPattern("###"); while( (idx = result.find(rxp)) > -1) result.replace(idx, 3, "");
  rxp.setPattern("\\.;"); while( (idx = result.find(rxp)) > -1) result.replace(idx, 2, ".");
  rxp.setPattern(",[a-zA-Z-]"); while( (idx = result.find(rxp)) > -1) result.replace(idx, 1, ", ");
  rxp.setPattern("[-]\\."); while( (idx = result.find(rxp)) > -1) result.replace(idx+1, 1, "");

  rxp2.setPattern("prep\\.|pron\\.|conj\\.|abbr\\.|Pref\\.|Phr\\.|pro\\.|adv\\.|adj\\.|pre\\.|con\\.|in\\.|pt\\.|pp\\.|fr\\.|pl\\.|SW\\.|vi\\.|vt\\.|C\\.|S\\.|A\\.|F\\.|n\\.|p\\.|v\\.");
  rxp.setPattern("S\\.|A\\.|SW\\.");
  result = result.simplifyWhiteSpace();

  enum {lNone=0, lType, lMeaning, lAnSynEx  };
  int lastSect = lNone;

  bool isAntSynEx = false;

    idx = 0; tmp = result.section(' ', idx, idx);
    if (tmp.isEmpty()) tempStr = "<br>" + result;

    isAntSynEx = (tmp.find(rxp) != -1);

    while (!tmp.isEmpty()) {
      if (firstItem) {
        if (tmp[0] == '(') {
          tempStr += (tmp + ((tmp.find(')') == -1) ? " " : "<br>") );
          while (tmp.find(')') == -1) {
            idx++; tmp = result.section(' ', idx, idx);
            tempStr += (tmp + ((tmp.find(')') == -1) ? " " : "<br>") );
          }
        }
        else
          tempStr += ("<br>" + tmp + " ");
        firstItem = false;
      }
      else {
        if (!isAntSynEx && (tmp.find(rxp2) != -1)) {
          isAntSynEx = (tmp.find(rxp) != -1);
          if (!lineStr.isEmpty()) {
            tempStr += (lineStr + ((isAntSynEx)||(lastSect!=lType) ? "<br>":" ") );
          }
          lineStr  = tmp + " ";
          lastSect = lType;
        }
        else {
          lineStr += (tmp + " ");
          if (isAntSynEx) lastSect = lType; else lastSect = lNone;
        }
      }
      idx++; tmp = result.section(' ', idx, idx);
      if (tmp.find(rxp) != -1) isAntSynEx = false;
    }
    tempStr += lineStr;
  //rxp.setPattern(""); while( (idx = result.find(rxp)) > -1) result.replace(idx+1, 2, " ");

  return tempStr.simplifyWhiteSpace();
}

*/
