/***************************************************************************
                          dictresview.cpp  -  description
                             -------------------
    begin                : . .. 25 2002
    copyright            : (C) 2002 by Thanomsub Noppaburana
    email                : donga_n@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qclipboard.h>
#include <qtimer.h>

#include <kcursor.h>
#include <kpopupmenu.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <khtml_events.h>
#include <khtmlview.h>
#include <kapplication.h>
#include <kiconloader.h>

#include "dictengine.h"
#include "dictactions.h"
#include "dictconfig.h"
#include "dictresview.h"



// ***************************************************************
//                   Class CBrowseData Implementation
// ***************************************************************

CBrowseData::CBrowseData(const QString &NhtmlStr, const QString &NwordRequest)
  : htmlStr(NhtmlStr), wordRequest(NwordRequest), xPos(0), yPos(0)
{
}


// ***************************************************************
//                   Class CHTMLPart Implementation
// ***************************************************************

CHTMLPart::CHTMLPart(QWidget *parent, const char *name)
  : KHTMLPart(parent, name)
{
}


CHTMLPart::~CHTMLPart()
{
}


void CHTMLPart::khtmlMouseReleaseEvent(khtml::MouseReleaseEvent *event)
{
  if (event->qmouseEvent()->button()==MidButton)
    emit(middleButtonClicked());
  else
    KHTMLPart::khtmlMouseReleaseEvent(event);
}



// ***************************************************************
//                   Class CDictView Implementation
// ***************************************************************

CResultView::CResultView(QWidget *parent)
  : QVBox(parent), actBackBTN(0L), actForwardBTN(0L),
    actKeyCMB(0L), browsePos(0)
{
  browseList.setAutoDelete(true);
  
  htmlView=new CHTMLPart(this);
  htmlView->setDNDEnabled(false);
  htmlView->setJScriptEnabled(false);
  htmlView->setJavaEnabled(false);
  htmlView->setURLCursor(KCursor::handCursor());

  setFocusPolicy(QWidget::NoFocus);

  popupMenu = new KPopupMenu(this);
  connect(htmlView, SIGNAL(popupMenu(const QString &, const QPoint &)), this, SLOT(slotBuildPopupMenu(const QString &, const QPoint &)));
  connect(htmlView->browserExtension(),SIGNAL(enableAction(const char *,bool)),this,SLOT(slotEnableAction(const char *,bool)));

  optionChanged();
  
  connect(dictEngine, SIGNAL(resultReady(const QString &)), SLOT(slotResultReady(const QString &)));
}


CResultView::~CResultView()
{
}


void CResultView::setActions(KToolBarPopupAction* actBack, KToolBarPopupAction* actForward, CActComboBox* actKeyCombo)
{
  actBackBTN = actBack;
  connect(actBackBTN->popupMenu(), SIGNAL(activated(int)), SLOT(slotBrowseBack(int)));
  actForwardBTN = actForward;
  connect(actForwardBTN->popupMenu(), SIGNAL(activated(int)), SLOT(slotBrowseForward(int)));
  actKeyCMB = actKeyCombo;
}


void CResultView::begin(const KURL &url, int xOffset, int yOffset)
{
  htmlView->begin(url, xOffset, yOffset);
}


void CResultView::write(const char *str)
{
  htmlView->write(str);
}


void CResultView::write(const QString &str)
{
  htmlView->write(str);
}


void CResultView::end()
{
  htmlView->end();
}


bool CResultView::canBrowseBack()
{
  return (browsePos > 0)? true:false;
}


bool CResultView::canBrowseForward()
{
  return (browsePos + 1 < browseList.count()) ? true:false;
}


void CResultView::generateHeader()
{
  HTMLheader = QString("<html><head><style type=\"text/css\">\n");

  HTMLheader += QString("table { margin-left: 1.5mm; margin-top: 2mm; margin-bottom: 2mm;}\n");
  HTMLheader += QString("div.definition { margin-left: 5mm; margin-top: 0mm; margin-bottom: 0mm; }\n");

  //////////  Vocab
  HTMLheader += QString("span.vocab { font-family:%1; font-size: %2px; color: %3;").
      arg(global->font(FVocab).family()).
      arg(global->font(FVocab).pointSize()+3).
      arg(global->color(CVocab).name())
  ;
  if (global->font(FVocab).italic())
    HTMLheader += QString("font-style:italic; ");
  if (global->font(FVocab).bold())
    HTMLheader += QString("font-weight:bold; ");
  HTMLheader += QString("}\n");
  //////////

  //////////  Spell word
  HTMLheader += QString("span.spell { font-family:%1; font-size: %2px; color: %3;").
      arg(global->font(FSpell).family()).
      arg(global->font(FSpell).pointSize()+3).
      arg(global->color(CSpell).name())
  ;
  if (global->font(FSpell).italic())
    HTMLheader += QString("font-style:italic; ");
  if (global->font(FSpell).bold())
    HTMLheader += QString("font-weight:bold; ");
  HTMLheader += QString("}\n");
  //////////

  //////////  Vocab Type
  HTMLheader += QString("span.vocabtype { font-family:%1; font-size: %2px; color: %3;").
      arg(global->font(FVocabType).family()).
      arg(global->font(FVocabType).pointSize()+3).
      arg(global->color(CVocabType).name())
  ;
  if (global->font(FVocabType).italic()) HTMLheader += QString("font-style:italic; ");
  if (global->font(FVocabType).bold())   HTMLheader += QString("font-weight:bold; ");
  HTMLheader += QString("}\n");
  //////////

  //////////  Meaning word
  HTMLheader += QString("span.meaning { font-family:%1; font-size: %2px; color: %3;").
      arg(global->font(FMeaning).family()).
      arg(global->font(FMeaning).pointSize()+3).
      arg(global->color(CMeaning).name())
  ;
  if (global->font(FMeaning).italic())
    HTMLheader += QString("font-style:italic; ");
  if (global->font(FMeaning).bold())
    HTMLheader += QString("font-weight:bold; ");
  HTMLheader += QString("}\n");
  //////////

  //////////   Pre
  HTMLheader += QString("pre { font-family:%1; font-size:%2px; ").arg(global->font(FNormalText).family()).arg(global->font(FNormalText).pointSize());
  if (global->font(FNormalText).italic())
    HTMLheader += QString("font-style:italic; ");
  if (global->font(FNormalText).bold())
    HTMLheader += QString("font-weight:bold; ");
  HTMLheader += QString("}\n");
  //////////

  HTMLheader += QString("</style></head>\n");
}


void CResultView::optionChanged()
{
  slotSaveCurrentPos();
  generateHeader();
  showResult();
}


void CResultView::showResult()
{
  begin();
  if (browseList.isEmpty()) {
    write(HTMLheader + "<body></body></html>");
  } else {
    CBrowseData* brw = browseList.at(browsePos);
    QString result(HTMLheader + "<body>" + brw->htmlStr + "</body></html>");
    write(result);
    emit(renderComplete(result));
  }
  end();
}


// ********* CResultView Slot implement ********** //

void CResultView::slotPrintResult()
{
  htmlView->view()->print();
}


void CResultView::slotCopySelection()
{
  kapp->clipboard()->setText(htmlView->selectedText());
}


void CResultView::slotSaveCurrentPos()
{
  if (! browseList.isEmpty()) {
    browseList.at(browsePos)->xPos = htmlView->view()->contentsX();
    browseList.at(browsePos)->yPos = htmlView->view()->contentsY();
  }
}


void CResultView::slotSelectAll()
{
  htmlView->selectAll();
}

void CResultView::slotEnableAction(const char * name, bool enabled)
{
  if (!strcmp(name, "copy")) emit(enableCopy(enabled));
}


void CResultView::slotBrowseBack()
{
  if (canBrowseBack()) {
    slotSaveCurrentPos();
    actKeyCMB->setEditText(browseList.at(--browsePos)->wordRequest);
    showResult();
    slotUpdateBrowseActions();
  }
}


void CResultView::slotBrowseForward()
{
  if (canBrowseForward()) {
    slotSaveCurrentPos();
    actKeyCMB->setEditText(browseList.at(++browsePos)->wordRequest);
    showResult();
    slotUpdateBrowseActions();
  }
}


void CResultView::slotShowFindDialog()
{
  KAction *act = htmlView->actionCollection()->action("find");
  if (act) act->activate();
}


// ** protected slots ** //

void CResultView::slotBrowseBack(int idx)
{
  int x = browsePos - idx;
  if (x >= 0) {
    slotSaveCurrentPos();
    browsePos = x;
    actKeyCMB->setEditText(browseList.at(browsePos)->wordRequest);
    showResult();
    QTimer::singleShot(0, this, SLOT(slotUpdateBrowseActions()));  // don't clear the menu in this slot
  }
}


void CResultView::slotBrowseForward(int idx)
{
  int x = browsePos + idx;
  if (x < (int)(browseList.count())) {
    slotSaveCurrentPos();
    browsePos = x;
    actKeyCMB->setEditText(browseList.at(browsePos)->wordRequest);
    showResult();
    QTimer::singleShot(0, this, SLOT(slotUpdateBrowseActions()));
  }
}


void CResultView::slotResultReady(const QString &result)
{
  CBrowseData* brw = new CBrowseData(result, dictEngine->keyword());

  if (browseList.isEmpty()) {
    browsePos = 0;
    browseList.append(brw);
  } else {
    slotSaveCurrentPos();
    while (browseList.count() > browsePos + 1) browseList.removeLast();
    browseList.append(brw);
    browsePos++;
    while (browseList.count() > global->maxBrowseEntry) {
      browseList.removeFirst();
      browsePos--;
    }
  }

  showResult();
  emit(enablePrint());
  slotUpdateBrowseActions();
}


void CResultView::slotUpdateBrowseActions()
{
  // ** Back **
  if (canBrowseBack()) {
    actBackBTN->setEnabled(true);
    if (browseList.at(browsePos-1)->wordRequest.isEmpty())
      actBackBTN->setText(i18n("&Back: Information"));
    else
      actBackBTN->setText(i18n("&Back: '%1'").arg(getShortString(browseList.at(browsePos-1)->wordRequest,25)));

    actBackBTN->popupMenu()->clear();
    int i = browsePos-1;
    int num = 1;
    QString s;
    while ((i>=0) && (num<=10)) {
      s = browseList.at(i)->wordRequest;
      if (s.isEmpty()) s = i18n("Information");
      actBackBTN->popupMenu()->insertItem(s, num);
      num++;
      i--;
    }
  } else {
    actBackBTN->setEnabled(false);
    actBackBTN->setText(i18n("&Back"));
    actBackBTN->popupMenu()->clear();
  }

  // ** Forward **
  if (canBrowseForward()) {
    actForwardBTN->setEnabled(true);
    if (browseList.at(browsePos+1)->wordRequest.isEmpty())
      actForwardBTN->setText(i18n("&Forward: Information"));
    else
      actForwardBTN->setText(i18n("&Forward: '%1'").arg(getShortString(browseList.at(browsePos+1)->wordRequest,25)));

    actForwardBTN->popupMenu()->clear();
    int i = browsePos+1;
    int num = 1;
    QString s;
    while ((i<(int)(browseList.count())) && (num<=10)) {
      s = browseList.at(i)->wordRequest;
      if (s.isEmpty()) s = i18n("Information");
      actForwardBTN->popupMenu()->insertItem(s, num);
      num++;
      i++;
    }
  } else {
    actForwardBTN->setEnabled(false);
    actForwardBTN->setText(i18n("&Forward"));
    actForwardBTN->popupMenu()->clear();
  }
}


void CResultView::slotBuildPopupMenu(const QString &url, const QPoint &point)
{
  popupMenu->clear();

  if (!url.isEmpty()) {                // menuitem if mouse is over link
    KURL u(url);
    QString type = u.host();
    popupLink = u.prettyURL();
    popupLink.remove(0, type.length() + 8);

    if (type.length()) {             // valid url
      if (type == "define") {
        popupMenu->insertItem(i18n("Define &Synonym"), this, SLOT(popupDefineLink()));
        popupMenu->insertItem(i18n("M&atch Synonym"), this, SLOT(popupMatchLink()));
        popupMenu->insertSeparator();
      }
      if(type == "dbinfo") {
        popupMenu->insertItem(i18n("D&atabase Information"),this,SLOT(popupDbInfo()));
        popupMenu->insertSeparator();
      }
      if(type == "realhttp") {
        popupLink.prepend("http://");
        popupMenu->insertItem(SmallIcon("fileopen"), i18n("&Open Link"), this, SLOT(popupOpenLink()));
        popupMenu->insertSeparator();
      }
      if(type=="realftp") {
        popupLink.prepend("ftp://");
        popupMenu->insertItem(SmallIcon("fileopen"), i18n("&Open Link"), this, SLOT(popupOpenLink()));
        popupMenu->insertSeparator();
      }
    }
  }

  if (htmlView->hasSelection()) {
    popupSelect = htmlView->selectedText();
    //////////////////
    popupMenu->insertItem(i18n("&Find Selected text"), this, SLOT(slotFindWordOnClipboard()));
    //////////////////
    popupMenu->insertSeparator();
  } else {
    kapp->clipboard()->setSelectionMode(false);
    QString text = kapp->clipboard()->text();
    if (text.isEmpty()) {
      kapp->clipboard()->setSelectionMode(true);
      text = kapp->clipboard()->text();
    }
    if (!text.isEmpty()) {
      popupSelect = QApplication::clipboard()->text();
      popupMenu->insertItem(SmallIcon("kfind"/*"find_clipboard"*/), i18n("&Find Word from Clipboard"), this, SLOT(slotFindWordOnClipboard()));
      popupMenu->insertSeparator();
    }
  }

  int ID;
  if (canBrowseBack()) {
    if (browseList.at(browsePos-1)->wordRequest.isEmpty())
      popupMenu->insertItem(SmallIcon("back"), i18n("&Back: Information"), this, SLOT(slotBrowseBack()));
    else
      popupMenu->insertItem(SmallIcon("back"), i18n("&Back: '%1'").arg(getShortString(browseList.at(browsePos-1)->wordRequest, 25)), this, SLOT(slotBrowseBack()));
  } else {
    ID = popupMenu->insertItem(SmallIcon("back"), i18n("&Back"), this, SLOT(slotBrowseBack()));
    popupMenu->setItemEnabled(ID, false);
  }

  if (canBrowseForward()) {
    if (browseList.at(browsePos+1)->wordRequest.isEmpty())
      popupMenu->insertItem(SmallIcon("forward"), i18n("&Forward: Information"), this, SLOT(slotBrowseForward()));
    else
      popupMenu->insertItem(SmallIcon("forward"), i18n("&Forward: '%1'").arg(getShortString(browseList.at(browsePos+1)->wordRequest, 25)), this, SLOT(slotBrowseForward()));
  } else {
    ID = popupMenu->insertItem(SmallIcon("forward"), i18n("&Forward"),this,SLOT(slotBrowseForward()));
    popupMenu->setItemEnabled(ID, false);
  }

  popupMenu->popup(point);
}


//-----------------------
#include "dictresview.moc"
