/***************************************************************************
                          dictsystemtray.cpp  -  description
                             -------------------
    begin                : . .. 27 2002
    copyright            : (C) 2002 by Thanomsub Noppaburana
    email                : donga_n@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <qcursor.h>
#include <qmessagebox.h>
#include <qtooltip.h>
#include <qtimer.h>
#include <qwhatsthis.h>

#include <klocale.h>
#include <kcursor.h>
#include <kiconloader.h>
#include <kpopupmenu.h>

#include <kwinmodule.h>

#include <kmainwindow.h>
#include <kapp.h>
#include <kdebug.h>

#include "kdictthai.h"
#include "dictresview.h"
#include "dictconfig.h"
#include "dictsystemtray.h"


// ***************************************************************
//             Class CDictSystemTray Implementation
// ***************************************************************

CDictSystemTray::CDictSystemTray(QWidget* parent, const char* name)
        : KSystemTray(parent, name)
{
  //trayMenu = contextMenu();
  trayMenu = new KPopupMenu(this);
  trayMenu->insertTitle(SmallIcon("kdictthai", KIcon::SizeSmall),  "KDictThai");

  globalKeys = new KGlobalAccel(this);
  KGlobalAccel* keys = globalKeys;
  #include "dictkeybindings.inc"
  globalKeys->updateConnections();
    
  watchItem = trayMenu->insertItem(SmallIcon("klipper", KIcon::SizeSmall), i18n("&Watch Clipboard"), this, SLOT(slotToggleWatchClipboard()));
  trayMenu->insertItem(SmallIcon("kfind", KIcon::SizeSmall), i18n("&Find Word on Clipboard"), this, SLOT(slotSearchOnClipboard()), globalKeys->shortcut("Find Word on Clipboard"));
  trayMenu->insertSeparator();
  
  restoreItem = trayMenu->insertItem(i18n("Restore"), this, SLOT(switchRestore()));

  // int configMenu =
  trayMenu->insertItem(SmallIcon("configure", KIcon::SizeSmall), i18n("&Preferences..."), this, SLOT(slotConfig()));
  trayMenu->insertSeparator();
  trayMenu->insertItem(SmallIcon("exit", KIcon::SizeSmall), i18n("&Quit"), this, SLOT(confirmQuit()));
  connect(trayMenu, SIGNAL(aboutToShow()), this, SLOT(prepareMenu()));

  trayMenu->setItemChecked(watchItem, global->watchClipboard);

  setBackgroundMode(X11ParentRelative);

  QToolTip::add(this, i18n("KDictThai"));
}


CDictSystemTray::~CDictSystemTray()
{
}


void CDictSystemTray::assignResultView(CResultView *resView)
{
  connect(resView, SIGNAL(renderComplete(QString &)), this, SLOT(slotResultReady(QString &)));
}


void CDictSystemTray::prepareMenu()
{
  if (parentWidget()->isVisible())
    trayMenu->changeItem(restoreItem, i18n("Minimize"));
  else
    trayMenu->changeItem(restoreItem, i18n("Restore"));

  if (global->watchClipboard != trayMenu->isItemChecked(watchItem))
    trayMenu->setItemChecked(watchItem, global->watchClipboard);
}


// ********* CDictSystemTray Slot implement ********** //

void CDictSystemTray::mousePressEvent(QMouseEvent *ev)
{
  if (ev->button() == RightButton) {
    trayMenu->exec(QCursor::pos());
  } else {
    KSystemTray::mousePressEvent(ev);
  }
}


void CDictSystemTray::mouseReleaseEvent(QMouseEvent *ev)
{
  KSystemTray::mouseReleaseEvent(ev);
}


void CDictSystemTray::switchRestore()
{
  if (parentWidget()->isVisible()) parentWidget()->hide(); else parentWidget()->show();
}


void CDictSystemTray::slotConfig()
{
  ((KDictThai *) parentWidget())->slotShowOptionDialog();
}


void CDictSystemTray::slotToggleWatchClipboard()
{
  ((KDictThai *) parentWidget())->slotToggleWatchClipboard();
  trayMenu->setItemChecked(watchItem, global->watchClipboard);
}
  

void CDictSystemTray::slotSearchOnClipboard()
{
  ((KDictThai *) parentWidget())->slotFindWordOnClipboard();
}


void CDictSystemTray::slotResultReady(QString &result)
{
  if ( ! ((KDictThai *)parentWidget())->isHidden() ) return;
  QWhatsThis::display(result);
}


void CDictSystemTray::confirmQuit() {
  if (QMessageBox::information(this,
                   i18n("Exit from KDictThai ?"),
                   i18n("Do you really want to leave KDictThai ?"),
                   i18n("&Yes"), i18n("&No"), 0, 0, 1) == 0
     )
  {
    KDictThai *parent = (KDictThai *) parentWidget();
    parent->saveSettings();
    kapp->quit();
  }
}


//===========================
#include "dictsystemtray.moc"
