/***************************************************************************
                          kdictthai.cpp  -  description
                             -------------------
    begin                : . .. 22 20:37:33 ICT 2002
    copyright            : (C) 2002 by Thanomsub Noppaburana
    email                : donga_n@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <dcopclient.h>
#include <qtooltip.h>
#include <qclipboard.h>
#include <qwhatsthis.h>

#include <kstdaction.h>
#include <kstdaccel.h>
#include <kkeydialog.h>
#include <kedittoolbar.h>
#include <kapplication.h>
#include <kstatusbar.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kwin.h>
#include <kdebug.h>

#include "dictsystemtray.h"
#include "dictengine.h"
#include "dictconfig.h"
#include "dictactions.h"
#include "dictresview.h"
#include "dictwordlist.h"
#include "watchclipboard.h"
#include "kdictthai.h"


CGlobalData *global;
CDictEngine *dictEngine = NULL;

// ***************************************************************
// cut a QString and add "..."
QString getShortString(QString str,unsigned int length)
{
  if (str.length()>length) {
    str.truncate(length-3);
    str.append("...");
  }
  return str;
}
// ***************************************************************



// ***************************************************************
//                   Class KDictThai Implementation
// ***************************************************************

KDictThai::KDictThai(QWidget* parent, const char *name, WFlags f)
  : KMainWindow(parent, name, f), DCOPObject("KDictThaiIface"), splitter(0), optionDialog(0)
{
  global = new CGlobalData();
  global->readConfig();

  watchCliboard = new CWatchClipboard();
  connect(watchCliboard, SIGNAL(clipboardChanged(QString)), SLOT(slotClipboardChanged(QString)));

  dictEngine = new CDictEngine();
  connect(dictEngine, SIGNAL(searchComplete()), SLOT(slotSearchComplete()));

  resultView = new CResultView(this);
  connect(resultView, SIGNAL(enableCopy(bool)), SLOT(slotEnableCopy(bool)));
  connect(resultView, SIGNAL(enablePrint()), SLOT(slotEnablePrint()));
  setCentralWidget(resultView);

  wordList = new CWordList(this);
  connect(wordList, SIGNAL(highlighted(const QString &)), SLOT(slotHighlighted(const QString &)));

  setupWordList();
  setupStatusBar();
  setupActions();
  recreateGUI();
  buildHistoryMenu();

  actToggleWordList->setChecked(global->showWordList);
  
  resize(global->userSize.width(), global->userSize.height());

  applyMainWindowSettings(KGlobal::config(), "WindowSettings");
  actToggleToolbar->setChecked(!toolBar()->isHidden());
  actToggleStatusbar->setChecked(!statusBar()->isHidden());

  actKeyCMB->setFocus();

  dock = new CDictSystemTray(this, "dock");
  dock->setPixmap(MainBarIcon("kdictthai", 22));
  QToolTip::add(dock, "kdictthai");

  actToggleDock->setChecked(global->showDockIcon);
  if (global->showDockIcon) {
    dock->show();
  } else {
    global->showMainWindow = true;
    dock->hide();
  }
  dock->assignResultView(resultView);

  QWhatsThis::add(resultView, i18n("This is Meaning area of vocab word"));
  QWhatsThis::add(wordList, i18n("This is area of related Vocabs List"));

  if (global->watchClipboard) watchCliboard->startWatch();
}


KDictThai::~KDictThai()
{
  delete dictEngine;
  delete watchCliboard;
  delete global;
}


// ********* DCOP Interface ********* //

void KDictThai::quit()
{
  kdDebug(5004) << "Quit: Sender = " << sender()->className() << endl;
  kapp->closeAllWindows();
}


void KDictThai::makeActiveWindow()
{
  KWin::setOnDesktop(winId(), KWin::currentDesktop());
  KWin::setActiveWindow(winId());
}



// *********  ********* //

void KDictThai::saveSettings()
{
  saveMainWindowSettings(KGlobal::config(), "WindowSettings");
  global->wordListSize = wordList->width();
  if (splitter) global->splitterSize = splitter->sizes();
  global->showMainWindow = this->isVisible();
  global->writeConfig();
}


bool KDictThai::queryClose()
{
  saveSettings();
  if ((sender() != NULL) && (sender()->className()=="KDictApplication") && global->showDockIcon) {
    hide();
    kdDebug(5004) << "\tHide... " << endl;
    return false;
  }
  kdDebug(5004) << "\tClose... " << endl;
  return true;
}


void KDictThai::setupActions()
{
  // ** file menu... ** //
  actPrint = KStdAction::print(resultView, SLOT(slotPrintResult()), actionCollection());
  actPrint->setEnabled(false);
  KStdAction::quit(kapp, SLOT(closeAllWindows()), actionCollection());

  // ** edit menu... ** //
  KStdAction::find(resultView, SLOT(slotShowFindDialog()), actionCollection());
  actCopy = KStdAction::copy(resultView, SLOT(slotCopySelection()), actionCollection());
  actCopy->setEnabled(false);
  KStdAction::selectAll(resultView, SLOT(slotSelectAll()), actionCollection());
  new KAction(i18n("&Find Word from Clipboard"), "kfind", 0 , this,
              SLOT(slotFindWordOnClipboard()), actionCollection(), "find_clipboard");

  // ** history menu... ** //
  actBrowseBack = new KToolBarPopupAction(i18n("&Back"), "back", KStdAccel::shortcut(KStdAccel::Back),
                            resultView, SLOT(slotBrowseBack()), actionCollection(), "browse_back");
  actBrowseBack->setDelayed(true);
  actBrowseBack->setStickyMenu(false);
  actBrowseBack->setEnabled(false);
  actBrowseForward = new KToolBarPopupAction(i18n("&Forward"), "forward", KStdAccel::shortcut(KStdAccel::Forward),
                            resultView, SLOT(slotBrowseForward()), actionCollection(), "browse_forward");
  actBrowseForward->setDelayed(true);
  actBrowseForward->setStickyMenu(false);
  actBrowseForward->setEnabled(false);
  new KAction(i18n("&Clear History"), "editclear", 0 , this, SLOT(slotClearWordHistory()),
              actionCollection(), "clear_history");
  
  // ** settings menu... ** //
  actToggleToolbar   = KStdAction::showToolbar(this, SLOT(slotToggleToolbar()), actionCollection());
  actToggleStatusbar = KStdAction::showStatusbar(this, SLOT(slotToggleStatusbar()), actionCollection());
  actToggleWordList  = new KToggleAction(i18n("Show WordList"), 0 , this,
                                    SLOT(slotToggleWordList()), actionCollection(), "options_show_wordlist");
  actToggleDock      = new KToggleAction(i18n("Show Dock"), 0 , this,
                                    SLOT(slotToggleDockIcon()), actionCollection(), "show_dock");
  actToggleWatch     = new KToggleAction(i18n("Watch Clipboard"), SmallIcon("klipper", KIcon::SizeSmall), 0, this,
                                    SLOT(slotToggleWatchClipboard()), actionCollection(), "options_watch_clipboard");
  KStdAction::keyBindings(this, SLOT(slotConfigKeys()), actionCollection());
  KStdAction::configureToolbars(this, SLOT(slotConfigToolbar()), actionCollection());
  KStdAction::preferences(this, SLOT(slotShowOptionDialog()), actionCollection());

  actToggleWordList->setChecked(global->showWordList);
  actToggleDock->setChecked(global->showDockIcon);
  actToggleWatch->setChecked(global->watchClipboard);
  

  // ** toolbar... ** //
  new KAction(i18n("Clear Input"), "locationbar_erase", 0 , this,
              SLOT(slotClearInput()), actionCollection(), "keyword_clear");
  actKeyLB   = new CActLabel(i18n("&Word"), actionCollection(), "keyword_label");
  actKeyCMB  = new CActComboBox(i18n("Keyword"), actionCollection(), "keyword_combo",true,true);
  connect(actKeyCMB, SIGNAL(activated(const QString &)), SLOT(slotAssignKeyword(const QString&)));
  new KAction(i18n("Find"), "key_enter", 0 , this,
              SLOT(slotFindKeyword()), actionCollection(), "keyword_findbutton");

  resultView->setActions(actBrowseBack, actBrowseForward, actKeyCMB);
}


void KDictThai::setupStatusBar()
{
  statusBar()->insertItem(i18n(" Ready "), 0, 2);
  statusBar()->setItemAlignment(0, AlignLeft | AlignVCenter);
  QString dictCount(i18n("Dict contains: ") + QString::number(dictEngine->recordsCount())+i18n(" words"));
  statusBar()->insertItem(dictCount, 1, 0);
  statusBar()->setItemAlignment(1, AlignLeft | AlignVCenter);
}


void KDictThai::setupWordList()
{
  if (global->showWordList) {
    splitter = new QSplitter(QSplitter::Horizontal, this);
    splitter->setOpaqueResize(true);
    resultView->reparent(splitter, 0, resultView->pos(), true);
    wordList->reparent(splitter, 0, wordList->pos(), true);
    splitter->setResizeMode(wordList, QSplitter::KeepSize);
    setCentralWidget(splitter);
    wordList->show();
    splitter->show();
    splitter->setSizes(global->splitterSize);
  } else {
    resultView->reparent(this, 0, resultView->pos(), true);
    wordList->reparent(this, 0, wordList->pos(), true);
    wordList->hide();
    if (splitter != NULL) {
      delete splitter;
      splitter = NULL;
    }
    setCentralWidget(resultView);
    wordList->resize(QSize(global->wordListSize, wordList->height()));
  }
}

  
void KDictThai::recreateGUI()
{
  createGUI("kdictthaiui.rc", false);
  actKeyCMB->setList(global->wordHistory);
  actKeyCMB->clearEdit();
  actKeyLB->setBuddy(actKeyCMB->widget());
}


void KDictThai::buildHistoryMenu()
{
  unplugActionList("history_items");

  historyList.setAutoDelete(true);
  historyList.clear();

  unsigned int i = 0;
  while ((i < 10) && (i < global->wordHistory.count())) {
    historyList.append(
        new KAction(getShortString(global->wordHistory[i],70),
                    "flag", 0, this, SLOT(slotHistoryClicked()),
                    (QObject*)0, global->wordHistory[i].utf8().data()
        )
    );
    i++;
  }

  plugActionList("history_items", historyList);
}


void KDictThai::addWordToHistory()
{
  QString text(actKeyCMB->currentText());

  global->wordHistory.remove(text);                    // no double entrys
  global->wordHistory.prepend(text);                   // prepend new item
  while (global->wordHistory.count() > global->maxHistoryEntry)
    global->wordHistory.remove(global->wordHistory.fromLast());

  actKeyCMB->setList(global->wordHistory);
  actKeyCMB->setCurrentItem(0);
  buildHistoryMenu();
}


// ********* KDictThai Slot implement ********** //

void KDictThai::slotToggleStatusbar()
{
  if (statusBar()->isVisible()) statusBar()->hide(); else statusBar()->show();
}


void KDictThai::slotToggleToolbar()
{
  if(toolBar()->isVisible()) toolBar()->hide(); else toolBar()->show();
}


void KDictThai::slotToggleWordList()
{
  global->showWordList = !global->showWordList;
  setupWordList();
  actToggleWordList->setChecked(global->showWordList);
}


void KDictThai::slotToggleDockIcon()
{
  if (global->showDockIcon) {
    global->showDockIcon = false;
    dock->hide();
  } else {
    global->showDockIcon = true;
    dock->show();
  }
  actToggleDock->setChecked(global->showDockIcon);
}


void KDictThai::slotToggleWatchClipboard()
{
  global->watchClipboard = ! global->watchClipboard;
  actToggleWatch->setChecked(global->watchClipboard);
  if (global->watchClipboard) {
    watchCliboard->startWatch();
  }
  else {
    watchCliboard->stopWatch();
  }
}


void KDictThai::slotConfigKeys()
{
  KKeyDialog::configureKeys(actionCollection(), xmlFile(), true, this);
}


void KDictThai::slotConfigToolbar()
{
  saveMainWindowSettings(KGlobal::config(), "WindowSettings");
  KEditToolbar *dlg = new KEditToolbar(actionCollection(), "kdictthaiui.rc");

  if (dlg->exec()) {
    recreateGUI();
    applyMainWindowSettings(KGlobal::config(), "WindowSettings");
    buildHistoryMenu();
  }
  delete dlg;
}


void KDictThai::slotHistoryClicked()
{
  QCString name = sender()->name();
  if (!name.isEmpty()) slotAssignKeyword(QString::fromUtf8(name));
}


void KDictThai::slotShowOptionDialog()
{
  if (!optionDialog) {
    optionDialog = new COptionDialog(this);
    connect(optionDialog, SIGNAL(optionChanged()), this, SLOT(slotOptionChanged()));
    connect(optionDialog, SIGNAL(finished()), this, SLOT(slotHideOptionDialog()));
    optionDialog->show();
  } else {
    KWin::setActiveWindow(optionDialog->winId());
  }
}


void KDictThai::slotHideOptionDialog()
{
  if (optionDialog) {
    optionDialog->delayedDestruct();
    optionDialog = NULL;
  }
}


void KDictThai::slotOptionChanged()
{
  dictEngine->setMaxRelateWords(global->maxWordListEntry);
  resultView->optionChanged();
  setupWordList();
  if (global->showWordList) {
    wordList->clear();
    wordList->insertStringList(dictEngine->getRelateWordList());
    wordList->setCurrentItem(0);
  }
  watchCliboard->setInterval(global->watchInterval);
  if (global->watchClipboard != actToggleWatch->isChecked()) {
    global->watchClipboard = ! global->watchClipboard;
    slotToggleWatchClipboard();
  }
}


void KDictThai::slotClearInput()
{
  actKeyCMB->clearEdit();
  actKeyCMB->setFocus();
}


void KDictThai::slotClearWordHistory()
{
  global->wordHistory.clear();
  actKeyCMB->clear();
  buildHistoryMenu();
}


void KDictThai::slotEnableCopy(bool selected)
{
  actCopy->setEnabled(selected);
}


void KDictThai::slotEnablePrint()
{
  actPrint->setEnabled(true);
}


void KDictThai::slotAssignKeyword(const QString &key)
{
  actKeyCMB->setEditText(key);
  slotFindKeyword();
}


void KDictThai::slotFindKeyword()
{
  QString text(actKeyCMB->currentText());
  if (!text.isEmpty() && (text != currentKeyword)) {
    currentKeyword = text;
    dictEngine->searchKeyword(text);
  }
}


void KDictThai::slotFindWordOnClipboard()
{
  QString text = watchCliboard->getClipboardContent();
  slotAssignKeyword(text);
}


void KDictThai::slotHighlighted(const QString &keyword)
{
  if (keyword != currentKeyword) {
    currentKeyword = keyword;
    actKeyCMB->setEditText(keyword);
    dictEngine->searchWithNoSignal(keyword);
  }
}


void KDictThai::slotSearchComplete()
{
  actKeyCMB->setEditText(dictEngine->keyword());
  addWordToHistory();
  if (wordList->isVisible()) {
    wordList->clear();
    wordList->insertStringList(dictEngine->getRelateWordList());
    wordList->setCurrentItem(0);
  }

  //if (this->isHidden()) {
  //  QWhatsThis::display();
  //}
}


void KDictThai::slotClipboardChanged(QString content)
{
  if (currentKeyword != content) slotAssignKeyword(content);
}


//-----------------------
#include "kdictthai.moc"
